PARAMETERS l_cdxname, l_tagnum, l_whatinfo

PRIVATE ALL LIKE l_*

IF PARAMETERS() = 0
	WAIT "You must supply a CDX file name!" WINDOW TIMEOUT 5
	RETURN " "
ENDIF

IF AT(".",l_cdxname) = 0
	l_cdxname = l_cdxname+".cdx"
ENDIF

IF !FILE(l_cdxname)
	WAIT "Requested CDX file does not exist!" WINDOW TIMEOUT 5
	RETURN " "
ENDIF

IF TYPE('l_whatinfo') != "N"
	l_whatinfo = 1
ENDIF

IF TYPE('l_tagnum') != "N"
	l_tagnum = 0
ENDIF

l_handle 	= FOPEN(l_cdxname,0)

IF l_handle = -1
	WAIT "File is already opened.  Please close it so I can use it." WINDOW TIMEOUT 5
	RETURN " "
ENDIF

l_cdxhdr 	= CHR(0)+CHR(4)+CHR(0)+CHR(0)
l_offset		= 512
l_numtags 	= numoftags()

IF l_numtags < l_tagnum
	WAIT "There are only "+ALLTRIM(STR(l_numtags)) + " tags in this CDX file!" WINDOW TIMEOUT 5
	=FCLOSE(l_handle)
	RETURN " "
ENDIF

l_options	= 14
l_asc_des	= 502
l_hdrchar	= (l_tagnum*6)+4
l_taghdr		= CHR(0)+CHR(l_hdrchar)+CHR(0)+CHR(0)

DO CASE
	CASE l_whatinfo = 1
		l_retstr = l_numtags
	CASE l_whatinfo = 2
		l_retstr = uniquetag()
	CASE l_whatinfo = 3
		l_retstr = ascdestag()
ENDCASE

=FCLOSE(l_handle)

RETURN l_retstr


FUNCTION numoftags

l_pointer = FSEEK(l_handle,0,0)
l_rectype = FREAD(l_handle,4)

IF l_rectype != l_cdxhdr
	l_position = l_offset
	DO WHILE !FEOF(l_handle) AND (l_rectype != l_cdxhdr)
		l_pointer = FSEEK(l_handle,l_position,0)
		l_rectype = FREAD(l_handle,4)
		l_position = l_position+l_offset
	ENDDO
	l_position 	= l_pointer+(l_offset*2)
ELSE
	l_position 	= (l_offset*2)
ENDIF

l_pointer 	= FSEEK(l_handle,l_position,0)
l_rectype 	= FREAD(l_handle,4)
l_numoftags = ASC(SUBS(l_rectype,3,1))+ASC(RIGHT(l_rectype,1))

RETURN l_numoftags


FUNCTION uniquetag

=getcurtag()

l_pointer = FSEEK(l_handle,0,0)
l_rectype = FREAD(l_handle,4)

IF l_rectype != l_taghdr
	l_position = l_offset
	DO WHILE !FEOF(l_handle) AND (l_rectype != l_taghdr)
		l_pointer = FSEEK(l_handle,l_position,0)
		l_rectype = FREAD(l_handle,4)
		l_position = l_position+l_offset
	ENDDO
	l_position 	= l_pointer+l_options
ELSE
	l_position 	= l_options
ENDIF

l_pointer 	= FSEEK(l_handle,l_position,0)
l_rectype 	= ASC(FREAD(l_handle,1))
l_unique		= IIF(l_rectype = 97 OR l_rectype = 105,.T.,.F.)

RETURN l_unique


FUNCTION ascdestag

= getcurtag()

l_pointer = FSEEK(l_handle,0,0)
l_rectype = FREAD(l_handle,4)

IF l_rectype != l_taghdr
	l_position = l_offset
	DO WHILE !FEOF(l_handle) AND (l_rectype != l_taghdr)
		l_pointer = FSEEK(l_handle,l_position,0)
		l_rectype = FREAD(l_handle,4)
		l_position = l_position+l_offset
	ENDDO
	l_position 	= l_pointer+l_asc_des
ELSE
	l_position 	= l_asc_des
ENDIF

l_pointer 	= FSEEK(l_handle,l_position,0)
l_rectype 	= FREAD(l_handle,1)
l_ascending	= IIF(ASC(l_rectype) = 0,.T.,.F.)

RETURN l_ascending


* this function finds the duplicate count byte from which you
* can calculate the root node of the tag you're looking for
* this is necessary because FP just appends to the end
* of the cdx file whenever you delete a tag or recreate one

FUNCTION getcurtag
	* move pointer to beginning of file
	l_pointer = FSEEK(l_handle,0,0)
	l_rectype = FREAD(l_handle,4)

	* if the cdx hdr is not at the beginning of the file,
	* we have a problem.  let's look for it anyway
	
	IF l_rectype != l_cdxhdr
		l_position = l_offset
		DO WHILE !FEOF(l_handle) AND (l_rectype != l_cdxhdr)
			l_pointer = FSEEK(l_handle,l_position,0)
			l_rectype = FREAD(l_handle,4)
			l_position = l_position+l_offset
		ENDDO
		l_position 	= l_pointer+(l_offset*2)
	ELSE
		l_position 	= (l_offset*2)
	ENDIF
	
	* dimension an array to use to sort the tags in order to
	* select the one you want
	DIMENSION l_tagarray[l_numtags]

	* calculate the offset of the duplicate byte count for
	* the tag you're seeking
	l_numbyte	= l_position+23
	l_pointer 	= FSEEK(l_handle,l_numbyte,0)
	l_bytes	 	= ASC(FREAD(l_handle,1))

	* get all the tag offsets then sort them
	FOR l_count = 1 to l_numtags
		l_duplbyte	= l_pointer+2+(l_bytes*(l_count-1))
		l_byte 	= FSEEK(l_handle,l_duplbyte,0)
		l_tagarray[l_count] = ASC(FREAD(l_handle,1))
	ENDFOR

	=ASORT(l_tagarray)

	l_hdrchar	= l_tagarray[l_tagnum]+4
	l_taghdr		= CHR(0)+CHR(l_hdrchar)+CHR(0)+CHR(0)

RETURN